use Test::More tests => 7;

BEGIN {
use_ok( 'Flickr::API::Utils' );
use_ok( 'Flickr::API::Photos' );
use_ok( 'Flickr::API::People' );
use_ok( 'Flickr::API::Photosets' );
use_ok( 'Flickr::Photo' );
use_ok( 'Flickr::Photoset' );
use_ok( 'Flickr::Person' );
}
